#pragma once

#include "ofMain.h"
#include "ofxGPUFont.h"
#include "Atlas.h"
#include "Layer.h"
#include "Utils.h"

namespace ofxVariableLab {

class GPUFontLayer : public Layer {
    public:
        void setup(const LayerSettings & settings = LayerSettings()) override;
        void update() override;
        void draw(glm::vec3 position = glm::vec3(0, 0, 0)) override;
        void drawCharacter(const char character,
                           glm::vec3 position = glm::vec3(0, 0, 0),
                           glm::vec4 color = glm::vec4(1, 1, 1, 1),
                           float rotation = 0,
                           float scale = 1,
                           ofxVariableLab::FontVariation fontVariation = ofxVariableLab::FontVariation()) override;
        void setProps(const Props & props) override;
        const Props & getProps() const override;
        void clearPropsBuffer() override;
        void setId(const LayerID & id) override;
        const LayerID & getId() override;
        void setShader(shared_ptr <ofShader> _shader) override {
            ofLogError("GPUFontLayer") << "using OpenGL shaders directly, no ofShader implemented";
        }
        shared_ptr <ofShader> getShader() const override {
            ofLogError("GPUFontLayer") << "using OpenGL shaders directly, no ofShader implemented";
            return NULL; // evil?
        }

        const Type & getType() const override;
        void setVFlip(const VFlipState vFlip) override;

        //void setAtlas(shared_ptr <ofxGPUFont::Atlas> _atlas);
        //shared_ptr <ofxGPUFont::Atlas> getAtlas() const;

        //shared_ptr <ofxGPUFont::Atlas> atlas;

        LayerSettings settings;
        std::deque <Props> propsBuffer;
        /// \brief are props updated but not drawn yet
        bool isDirty = true;
        /// \brief hashed id, or just counting up
        string id = "";
        VFlipState vFlip = V_FLIP_UNKNOWN;

    private:
        Layer::Type type = GPUFONT;
};
}
