#pragma once

#include "ofMain.h"
#include "AtlasLayerCombo.h"
#include "MsdfAtlasLayerCombo.h"
#include "MsdfLayer.h"
#include "GPUFontAtlasLayerCombo.h"
#include "GPUFontLayer.h"
#include "Utils.h"

namespace ofxVariableLab {

class LayerComposition {
    public:
        void setup();
        void update();
        void draw() const;
        LayerID addLayer(const ComboIdentifier & identifier,
                         const string & text,
                         const std::vector <FontVariation> & variations);
        LayerID addLayer(const ComboIdentifier & identifier,
                         const Layer::Props & props,
                         const std::vector <FontVariation> & variations);
        void removeLayer(const LayerID & id); // TODO: make bool, to catch nonexisting
        shared_ptr <Layer> getLayer(const LayerID & layerID);
        const unordered_map <ComboIdentifier, shared_ptr <AtlasLayerCombo> > & getAtlasLayerCombos() const;
        void setVFlip(bool vFlip);

    private:
        VFlipState vFlipState = V_FLIP_UNKNOWN;
        unordered_map <LayerID, shared_ptr <Layer> > layers;
        unordered_map <ComboIdentifier, shared_ptr <AtlasLayerCombo> > atlasLayerCombos;

        /**
          * @brief lastGpuTextureOffset GL_TEXTURE0 + offset
          * we want to different textures for different fonts
          * this value is incremented/decremented when adding/removing GPUFont layers
          */
        int nextGpuTextureOffset = 0;
        //unordered_map <LayerIdentifier, shared_ptr <ofxVariableLab::Layer> > layers;
};

}
