#pragma once

#include <algorithm>
#include <string>
#include <set>
#include <glm/glm.hpp>

namespace ofxVariableLab {

struct FontVariationAxis {
    char * name;
    float minValue;
    float maxValue;
    float defaultValue;
};

struct FontVariation {
    char * name;
    float value;
};

template <class T>
inline void hash_combine(std::size_t & seed, const T & v){
    std::hash <T> hasher;
    seed ^= hasher(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
}
//template <class ForwardIt, class UnaryPredicate>
//ForwardIt remove_if(ForwardIt first, ForwardIt last, UnaryPredicate p){
//first = std::find_if(first, last, p);
//if(first != last){
//for(ForwardIt i = first; ++i != last;){
//if(!p(*i)){
//*first++ = std::move(*i);
//}
//}
//}
//return first;
//}

inline void removeDuplicateCharacters(std::string & str){
    std::set <char> chars;

    str.erase(
        std::remove_if(
            str.begin(),
            str.end(),
            [&chars](char i){
            // If encountered character, remove this one.
            if(chars.count(i)){
                return true;
            }

            // Otherwise, mark this character encountered and don't remove.
            chars.insert(i);
            return false;
        }
            ),
        str.end()
        );

}

enum VFlipState {
    V_FLIP_UNKNOWN = 0,
    V_FLIP_ON,
    V_FLIP_OFF
};
enum VFlipBehaviour {
    /// probe view matrix every frame
    V_FLIP_ALWAYS_AUTO = 0,
    /// probe view matrix once and remember
    V_FLIP_ONCE_AUTO,
    /// never probe and force off
    V_FLIP_ALWAYS_OFF,
    /// never probe and force on
    V_FLIP_ALWAYS_ON
};
void getVFlip(const VFlipBehaviour & vFlipBehaviour,
              const glm::mat4 & orientationMatrix,
              VFlipState & vFlip);

}
